////////////////////////////////////////////////////////////////////// 
// relativeFinder/RelativeParameters.cpp 
// (c) 2000-2010 Goncalo Abecasis
// 
// This file is distributed as part of the Goncalo source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile Goncalo.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Tuesday June 15, 2010
// 
 
#include "RelativeParameters.h"
#include "MerlinSimulator.h"
#include "MerlinCore.h"
#include "Pedigree.h"
#include "Houdini.h"
#include "Random.h"

int  RelativeParameters::maxMegabytes = 0;
bool RelativeParameters::trimPedigree = false;

// Specialized analyses
bool RelativeParameters::fitErrorRate = false;

// Pairing options
bool RelativeParameters::allPairs = true;
bool RelativeParameters::withinFamilyPairs = false;
int  RelativeParameters::job = 1;
int  RelativeParameters::jobCount = 1;

// Simulation parameters
bool RelativeParameters::simulateNull = false;
int  RelativeParameters::reruns = 0;

// Marker Clustering
String RelativeParameters::clusterFile;
double RelativeParameters::clusterDistance = _NAN_;
double RelativeParameters::clusterRsquared = _NAN_;

BEGIN_LONG_PARAMETERS(relativeLongParameters)
   LONG_PARAMETER_GROUP("Errors")
      LONG_DOUBLEPARAMETER("perAllele", &FuzzyInheritanceTree::perAlleleError)
      LONG_DOUBLEPARAMETER("perGenotype", &FuzzyInheritanceTree::perGenotypeError)
      LONG_PARAMETER("fit", &RelativeParameters::fitErrorRate)
   LONG_PARAMETER_GROUP("LD Clusters")
      LONG_STRINGPARAMETER("clusters", &RelativeParameters::clusterFile)
      LONG_DOUBLEPARAMETER("distance", &RelativeParameters::clusterDistance)
      LONG_DOUBLEPARAMETER("rsq", &RelativeParameters::clusterRsquared)
   LONG_PARAMETER_GROUP("Limits")
      LONG_INTPARAMETER("bits", &MerlinCore::maxBits)
      LONG_INTPARAMETER("megabytes", &RelativeParameters::maxMegabytes)
      LONG_INTPARAMETER("minutes", &MerlinCore::maxMinutes)
   LONG_PARAMETER_GROUP("Performance")
      LONG_PARAMETER("trim", &RelativeParameters::trimPedigree)
#ifndef __CHROMOSOME_X__
      LONG_PARAMETER("noCoupleBits", &Mantra::ignoreCoupleSymmetries)
#endif
//      LONG_PARAMETER("swap", &FamilyAnalysis::useSwap)
      LONG_PARAMETER("smallSwap", &MerlinCore::smallSwap)
   LONG_PARAMETER_GROUP("Pairs")
      EXCLUSIVE_PARAMETER("allPairs", &RelativeParameters::allPairs)
      EXCLUSIVE_PARAMETER("withinFamily", &RelativeParameters::withinFamilyPairs)
      LONG_INTPARAMETER("job", &RelativeParameters::job)
      LONG_INTPARAMETER("of", &RelativeParameters::jobCount)
//   LONG_PARAMETER_GROUP("Simulation")
//      LONG_PARAMETER("simulate", &RelativeParameters::simulateNull)
//      LONG_INTPARAMETER("reruns", &RelativeParameters::reruns)
//      LONG_STRINGPARAMETER("trait", &Simulator::model)
   BEGIN_LEGACY_PARAMETERS()
      LONG_DOUBLEPARAMETER("iTheta", &MerlinCore::imaginaryTheta)
END_LONG_PARAMETERS();

RelativeParameters::RelativeParameters() :
   pedigreeFile("merlin.ped"),
   dataFile("merlin.dat"),
   mapFile("merlin.map"),
   freqFile(""),
   alleleFrequencies(FREQ_ALL),
   random_seed(123456)
   {
   Add(new StringParameter('d', "Data File", dataFile));
   Add(new StringParameter('p', "Pedigree File", pedigreeFile));
   Add(new StringParameter('m', "Map File", mapFile));
   Add(new AlleleFrequencyParameter('f', "Allele Frequencies", alleleFrequencies, freqFile));
   Add(new IntParameter('r', "Random Seed", random_seed));
   Add(new LongParameters("Data Analysis Options", relativeLongParameters));
   }

void RelativeParameters::Read(int argc, char ** argv, int start)
   {
   ParameterList::Read(argc, argv, start);

   globalRandom.Reset(random_seed);

   // Memory usage limit should be positive
   if (maxMegabytes < 0)
      maxMegabytes = 0;

   // Clustering distance must be positive
   if (clusterDistance < 0.0)
      clusterDistance = _NAN_;

   // Clustering r-squared must be between 0 and 1
   if (clusterRsquared < 0.0)
      clusterRsquared = 0.0;

   if (clusterRsquared > 1.0)
      clusterRsquared = 1.0;

   // Limited range of options available with error modelling
   if (FuzzyInheritanceTree::perAlleleError != 0.0 ||
       FuzzyInheritanceTree::perGenotypeError != 0.0 ||
       fitErrorRate)
      {
      const char * reason = "The --%s option is disabled when error modeling is enabled";

      Enforce(clusterDistance, _NAN_, reason, "distance");
      Enforce(clusterRsquared, _NAN_, reason, "rsq");
      Enforce(clusterFile, "", reason, "clusters");
      }

   // Limited range of options when using imaginary components for recombination fraction
   if (MerlinCore::imaginaryTheta < 0.0)
      MerlinCore::imaginaryTheta = 0.0;

   if (reruns > 1)
      Enforce(simulateNull, true, "The --reruns option automatically enables the --simulate option\n");

   // Some basic calculations to convert megabyte memory usage
   // limit into maximum TreeNode count
   BasicTree::maxNodes = (1024 * 1024) / sizeof(TreeNode) * maxMegabytes;
   }


 
